/*Codigo para reproducir un teleindicados de via Adif en Arduino a
 * través de una pantalla ST7735 de 0.96" y 160x80px
 * Creado por Sergio Guevara
 * Version: 00 (25/10/2021)
 * Mas info y contacto en: https://www.forotrenes.com/foro/viewtopic.php?f=6&t=87140
*/

#include <Adafruit_GFX.h>     // Libreria básica de gráficos de Adafruit
#include <Adafruit_ST7735.h>  // Libreria de Adafruit para pantallas ST7735
#include <SPI.h>              // Protocolo SPI para comunicacion con la pantalla 
#include <Fonts/ADIFfont.h>   // Fuente con las letras en el formato usado por Adif

//Pueros a los que esta conectada la pantalla
#define TFT_CS        10 // Pin 8 de la pantalla
#define TFT_RST        9 // Pin 3 de la pantalla
#define TFT_DC         8 // Pin 4 de la pantalla

//Dimensiones y ubicacion de los paneles del teleindicador
#define HORA_X        3    // Posicion X panel Hora
#define HORA_Y        19   // Posicion Y panel Hora
#define HORA_PX       40   // Longitud en pixeles del panel Hora
#define TREN_X        51   // Posicion X panel Tren
#define TREN_Y        19   // Posicion Y panel Tren
#define TREN_PX       104  // Longitud en pixeles del panel Tren
#define DEST_X        3    // Posicion X panel Destino
#define DEST_Y        44   // Posicion Y panel Destino
#define DEST_PX       152  // Longitud en pixeles del panel Destino
#define OBSE_X        3    // Posicion X panel Observaciones
#define OBSE_Y        61   // Posicion Y panel Observaciones
#define OBSE_PX       152  // Longitud en pixeles del panel Observaciones

#define HEIGHT        13   // Altura en pixeles de todos los paneles
#define SCROLL_SP     30   // Velocidad de scroll en observaciones (en px/s)
#define OFFSET        9    // Distancia de la parte superior del panle a la inferior de las letras

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST); //Objeto Pantalla
GFXcanvas1 canvas(OBSE_PX, 11); //Imagen monocroma auxiliar usada para realizar el scroll evitando parpadeos.

// Funcion para convertir colores de RGB a 16bits
word ConvertRGB( byte B, byte G, byte R)
{
  return ( ~(((R & 0xF8) << 8) | ((G & 0xFC) << 3) | (B >> 3)) );
}

// Definicion de los colores que se van a usar
word color_fondo = ConvertRGB(0, 0, 0);
word color_panel = ConvertRGB(50, 50, 50);
word amarillo = ConvertRGB(255, 250, 35);
word rojo = ConvertRGB(255, 0, 0);

// Variables auxiliares
bool isClean = false;
long time_offset;
long currentTime;
int space = 0;

// Funcion básica para pintar toda l apantalla de color homogeneo y dibujar los cuatro paneles en los que 
// se muestra la informacion
void Dibujar_Fondo(word color_panel, word color_fondo){
  tft.fillScreen(color_panel);
  tft.fillRect(HORA_X, HORA_Y, HORA_PX, HEIGHT, color_fondo);
  tft.fillRect(TREN_X, TREN_Y, TREN_PX, HEIGHT, color_fondo);
  tft.fillRect(DEST_X, DEST_Y, DEST_PX, HEIGHT, color_fondo);
  tft.fillRect(OBSE_X, OBSE_Y, OBSE_PX, HEIGHT, color_fondo);
  }

// Funcion básica para escribir en el panel Hora
void Escribir_Hora(String hora){
  tft.setTextColor(amarillo);
  tft.setCursor(HORA_X, HORA_Y + OFFSET);
  tft.print(hora);
  }

// Funcion básica para escribir en el panel Tren
void Escribir_Tren(String tren){
  tft.setTextColor(amarillo);
  tft.setCursor(TREN_X, TREN_Y + OFFSET);
  tft.print(tren);
  }

// Funcion básica para escribir en el panel Destino
void Escribir_Destino(String destino){
  tft.setTextColor(amarillo);
  tft.setCursor(DEST_X, DEST_Y + OFFSET);
  tft.print(destino);
  }

// Funcion básica para escribir en el panel Observaciones
// - Si el texo tiene 19 caracteres o menos se muestra completo
// - Si tiene más, realiza Scroll
void Escribir_Observacion(String obs, word color, long time_offset){
  int longitud = obs.length();
  int scroll_px = longitud*8 + OBSE_PX;
  int space = 0;
  int new_space = 0;
  
  tft.setTextColor(color);

  if (longitud < 20){ //En este caso la observacion cabe entera y no necesita scroll
    tft.setCursor(OBSE_X, OBSE_Y + OFFSET);
    tft.print(obs);
  }else{ //En este caso se realiza el scroll
    new_space = ((currentTime - time_offset)*SCROLL_SP/1000)%scroll_px;
    if ((space != new_space)){
      canvas.fillScreen(false);
      canvas.setCursor(OBSE_PX - new_space,8);
      canvas.print(obs);
      tft.drawBitmap(OBSE_X, OBSE_Y+1, canvas.getBuffer(), OBSE_PX, 11, color, color_fondo);
      space = new_space;
    };
    };
}

// Funcion completa para mostrar toda la infromación de un tren
void Mostrar_Info(String hora, String tren, String destino, String obs){
  if((currentTime%1000) <= 100){ //No es necesario reescribir esto continuamente, así se evitan parpadeos
    Escribir_Hora(hora);
    Escribir_Tren(tren);
    Escribir_Destino(destino);
  };
  Escribir_Observacion(obs, amarillo, 0);
  }

// Funcion completa para mostrar toda la infromación de un tren indicando SALIDA INMEDIATA
void Mostrar_Salida_Inmediata(String hora, String tren, String destino, String obs){
  long int scroll_time = (obs.length()*8 + OBSE_PX)/SCROLL_SP * 1000;
  long int SI_time = 10000;
  long int loop_time = scroll_time + SI_time;
  if((currentTime/600)%2 == 0){
    Escribir_Hora(hora);
  }else{
    tft.fillRect(HORA_X, HORA_Y, HORA_PX, 13, color_fondo);
  } 
  if((currentTime%1000) <= 100){
    Escribir_Tren(tren);
    Escribir_Destino(destino);
  };
  if(currentTime%loop_time <= SI_time){
    if(isClean == false){
      tft.fillRect(OBSE_X, OBSE_Y, OBSE_PX, 13, color_fondo);
      isClean = true;}
    if((currentTime/500)%2 == 0){
      Escribir_Observacion("SALIDA INMEDIATA", amarillo, 0);
      time_offset = millis();
    }else{
      Escribir_Observacion("SALIDA INMEDIATA", rojo, 0);
      time_offset = millis();
    }
  }else{
    Escribir_Observacion(obs, rojo, time_offset);
    isClean = false;
  }
}
  

void setup() {
  tft.initR(INITR_MINI160x80);        // Inicializar la pantalla
  tft.setFont(&ADIFfont);             // Seleccionar la fuente de Adif
  tft.setRotation(1);                 // Poner la pantalla en Apaisado, poniendo 3 se gira 180º
  tft.setTextWrap(false);             // Desactivar el wrap para que no escriba una segunda linea si el texo no cabe
  canvas.setFont(&ADIFfont);          // Configuramos el canvas aquí para que no lo tenga que hacer en cada iteracion
  canvas.setTextWrap(false);
  Dibujar_Fondo(color_panel, color_fondo);  // Se dibuja el fondo y estamos listos
}

void loop() {
  currentTime = millis();
  Mostrar_Salida_Inmediata("22:45", "CERCANIAS", "MURCIA DEL CARMEN", "CON PARADA EN LORCA-SAN DIEGO, LA HOYA, TOTANA, ALHAMA DE MURCIA, LIBRILLA Y ALCANTARILLA");

  //A continuacion algunos otros ejemplos de informaciones. Si se quiere alternar entre varias hay que dibujar el fondo entre medias para evitar que queden superpuestas  
  //Dibujar_Fondo(color_panel, color_fondo);
  //Mostrar_Info("07:30", "AVE 03071", "BARCELONA SANTS", "SIN PARADAS");
  //Mostrar_Info("15:20", "AVANT 08158", "MADRID CHAMARTIN", "CON PARADA EN SEGOVIA GUIOMAR");
  //Mostrar_Info("", "", "", "BIENVENIDOS A LA ESTACION DE SEVILLA SANTA JUSTA");
}
